/**
 *
 * Gamelication.java
 * @author	Mark G. Tacchi (mtacchi@next.com) 
 * @version	0.8
 * Mar 27/1996
 *
 * Gamelication contains a thread which is used for distributing ticks to all
 * manager objects.  A tick is basically an instant in time and represents
 * the time granularity of the game.  
 *
 */

package com.next.gt;

import java.awt.*;
import java.util.Date;
import java.util.Vector;
//import java.awt.Graphics;
import java.awt.event.*;

abstract public class Gamelication extends java.awt.Panel implements Runnable  {

  //
  // This is the main thread, used for ticks.
  //
  public Thread runner= null;
  
  //
  // Gamelication is the master time keeper.
  //
  public long currentTickTimeMillis= System.currentTimeMillis();
  public long lastTickTimeMillis;
  
  //
  // Gamelication is the manager of the managers. 
  //
  public ActorManager		actorManager= new ActorManager(this);
  public DisplayManager		displayManager= new DisplayManager(this);
  public ScoreManager		scoreManager;
  //public EventManager		eventManager= new EventManager();
  public AudioManager		audioManager = new AudioManager();
  
  //
  // Store the directory of the codebase.
  //
  private String		codebaseDir;
  
  //
  // Sleep time for the thread.
  //
 public int				SLEEP_MILLIS= 50;



/**
 * Generate a random double between two doubles.
 */
public static double randBetween(double a, double b)
{
  double		val, scale, tmp;

  if (a > b) {
    tmp= a; a= b; b= tmp;
  } /*endif*/
  
  scale = (b-a);
  val = scale * Math.random();
  
  return (a + val);
  
} /*randBetween*/



/**
 * Initialize.  Takes the width and height of the window.
 */
public void init( int width, int height ) {
	init( width, height, "." );
} /*init*/

/**
 * Another routine for initialization, takes in a width, height, and a 
 * codebase.
 */
public void init( int width, int height, String codebaseDir ) {
	setSize( width, height );
	this.codebaseDir = codebaseDir;
}

/**
 * Start the thread.
 */
public void start() {
  if(runner==null) { //start new thread if it doesn't exist
    runner= new Thread(this); 
    runner.start();
    new Thread( audioManager ).start();
	//runner.setPriority (Thread.MAX_PRIORITY);
  } /*endif*/

} /*start*/



/**
 * Stop the thread.
 */
public void stop() {
  if (runner != null)
    runner.stop (); //kill thread when applet is stopped
  runner= null;
} /*stop*/



public long sleepMillis () {
  return SLEEP_MILLIS;
}



/**
 * Execution loop.  Used to call tick().
 */
public void run() {

  while (runner != null){
      try {
		Thread.sleep (sleepMillis ());
	  } catch(InterruptedException e){} //sleep
    tick();
  }/*endwhile*/
  
  runner= null;
  
} /*run*/



/**
 * Distribute tick to the ActorManager and update display.
 */
public void tick() { 

  lastTickTimeMillis= currentTickTimeMillis;
  currentTickTimeMillis= System.currentTimeMillis();

  actorManager.tick();
  
  repaint();
  
} /*tick*/


/**
 * Calculater the difference between the current tick and the last one.
 */
public double deltaTickTimeMillis() {
  return (double)(currentTickTimeMillis - lastTickTimeMillis);
} /*deltaTickTimeMillis*/


/**
 * Override update to avoid screen clears.
 */
public void update(Graphics g) {	
  paint(g);
} /*update*/



/**
 * Pass the Graphics onto the DisplayManager.
 */ 
public void paint(Graphics g) {
  displayManager.paint(g);
} /*paint*/



/**
 * Provide standard Gamelication info.
 */
public String getGamelicationInfo() {
  return "The Gamelication Toolkit\nVersion 0.1\nWritten by Jeremy Hutchins, jgh8962@cs.rit.edu\n\nYou are free to use, copy, and modify the source without restriction.  However, it is requested that the author is mentioned in any pertinent credit sections released with code developed with the Gamelet Toolkit.";
} /*getAppletInfo*/

/**
 * Get codebase for the directory of the Gamelication.
 */
public String getCodeBase() {
  return codebaseDir;
}

/**
 * Displays the current status of the current Gamelication.
 */
public void showStatus( String gamelicationStatus ) {
	System.out.println( gamelicationStatus );
}

/**
 * Get an image from a file.
 */
public Image getImage( String codeBase, String imagefile ) {
	return getToolkit().getImage( codeBase+"/"+imagefile );
}

/**
 * Play an .au file
 */
public void play( String codeBase, String auFile ) {
	audioManager.play( codeBase+auFile );
}

} /*Gamelication*/
